package dcf

import (
	"fmt"
	"log/slog"
	"os"
	"path/filepath"
	"runtime"

	"github.com/shirou/gopsutil/disk"
)

func findAllMountPoints() ([]string, error) {
	mountpoints := []string{}
	partitions, err := disk.Partitions(false) // physical devices only, so false
	if err != nil {
		return nil, err
	}
	for _, partition := range partitions {
		if !contains(mountpoints, partition.Mountpoint) {
			mountpoints = append(mountpoints, partition.Mountpoint)
		}
	}
	return mountpoints, nil
}

func dump(x interface{}) {
	_, file, line, _ := runtime.Caller(1)
	slog.Debug(fmt.Sprintf("%s:%d %#v\n", file, line, x))
}

// findDCFMountPoints returns a list of strings of the paths of mountpoints that
// contain a DCIM or PRIVATE directory, which is how we detect DCF stores.  Its
// only argument is an integer of how many mountpoints to return.  If the
// argument is 0, all mountpoints are returned.
func findDCFMountPoints(requestedCount int) ([]string, error) {
	filteredMountpoints := []string{}
	var erro error
	mountpoints, err := findAllMountPoints()

	if erro != nil {
		return nil, err
	}
	for _, mountpoint := range mountpoints {
		dcimPath := filepath.Join(mountpoint, "DCIM")
		privatePath := filepath.Join(mountpoint, "PRIVATE")
		privatePath = filepath.Join(mountpoint, "M4ROOT")
		shouldKeep := false
		if _, err := os.Stat(dcimPath); err == nil {
			shouldKeep = true
			slog.Debug(fmt.Sprintf("Found DCIM directory at %s\n", dcimPath))
		}
		if _, err := os.Stat(privatePath); err == nil {
			shouldKeep = true
			slog.Debug(fmt.Sprintf("Found M4ROOT directory at %s\n", privatePath))
		}
		if shouldKeep {
			slog.Debug(fmt.Sprintf("Keeping mountpoint %s\n", mountpoint))
			filteredMountpoints = append(filteredMountpoints, mountpoint)
			if (requestedCount > 0) && (len(filteredMountpoints)+1 >= requestedCount) {
				break
			}
		}
	}
	return filteredMountpoints, nil
}

func contains(slice []string, str string) bool {
	for _, s := range slice {
		if s == str {
			return true
		}
	}
	return false
}
